// Modul...: SSP_MASTER.C
// Version.: 1.0
// Compiler: KEIL-ARM
// Datum...: Dezember 2005
// Autor...: Udo Jrss / Ulrich Radig
//-----------------------------------------------------------------------------

#include "ssp_master.h"
#include "pal.h"

//-----------------------------------------------------------------------------

static pU16 ssp_write_buffer_ptr; // Zeiger auf Schreibpuffer
static U32 ssp_bytes_to_write; // Anzahl Datenbytes
//------------------------------------------------------------------------------
void FIQ_Handler(void) __fiq
{
  while ((SSPSR & TNF) && --ssp_bytes_to_write)
  {
    SSPDR = (U16)*ssp_write_buffer_ptr++; // Datenbyte in den Tx-FIFO schreiben
    //ssp_bytes_to_write--;
  }

  if (!ssp_bytes_to_write) // Wenn alle Bytes gesendet sind
  {
    SSPIMSC &= ~TXIM; // Interrupt Tx-FIFO "half empty" ausschalten
    //SSPCR1 = 0; // Schnittstelle deaktivieren
  }
}

//------------------------------------------------------------------------------
void ssp_master_init(U08 data_format)
{  
  PCONP |= PCSPI1; // SPI1 einschalten

  PINSEL1 |= SPI1_IOSET_MASK; // PIN Konfigurieren
  SSPCPSR = 2; // Clock Prescale Register setzten
  SSPCR0 = data_format | CPHA | FRF_SPI; // SPI-Format
  SSPCR1 = SSE; // Schnittstelle als Master aktivieren

  while (SSPSR & RNE) // Rx-FIFO leeren
  {
    SSPDR; // Dummy-Read
  }

  SSPIMSC = 0; // Alle SSP-Interrupt-Quellen ausschalten
  //sys_set_fiq(VIC_SSP); // SSP-IRQ setzen
}
//-----------------------------------------------------------------------------

void ssp_transfer(pU08 write_buffer_ptr)
{                      
  ssp_write_buffer_ptr = (pU16) write_buffer_ptr;
  ssp_bytes_to_write = PAL_CHAR_COLS / 2;

  SSPDR = 0x0;
  SSPDR = 0x0;

  SSPCR1 = SSE; // Schnittstelle als Master aktivieren
  while (!(SSPSR & TNF));
  SSPDR = 0x0;

  while (--ssp_bytes_to_write) // Tx-FIFO fllen
  {
  	while(!(SSPSR & TNF));
    SSPDR = ((*ssp_write_buffer_ptr & 0x00FF) << 8)+
			((*ssp_write_buffer_ptr & 0xFF00) >> 8); // Datenbyte in den Tx-FIFO schreiben 
	ssp_write_buffer_ptr++;
  }
  //SSPCR1 = SSE; Schnittstelle als Master aktivieren
  //SSPIMSC = TXIM;  Interrupt Tx-FIFO "half empty" einschalten
}